#!/usr/bin/env python

import math
import re
from builtins import len
import time

import os
import os.path
import sys

from OCC.Core import TopoDS
from OCC.Core.BRepPrimAPI import *
from OCC.Core.BRepFilletAPI import *
from OCC.Core.TopoDS import *
from OCC.Core.TopExp import * 
from OCC.Extend.TopologyUtils import TopologyExplorer
from OCC.Display.SimpleGui import *
from OCC.Display.OCCViewer import *
from OCC.Core.AIS import AIS_Shape, AIS_ColoredShape
from OCC.Core.Quantity import Quantity_Color, Quantity_TOC_RGB

display, start_display, add_menu, add_function_to_menu = init_display()

###=================
selected_surfaces=[]
listOne_surfaces=[]
listTwo_surfaces=[]
hidden_surfaces=[]
###=================

def box(event=None):
    global context 
    global aisShape
    global aBox
    global drawer
    global color0, transparency0
    color0=Quantity_Color(11./256.,157./256.,233./256.,0)
    transparency0=0.8
    display.EraseAll()
    aBox = BRepPrimAPI_MakeBox(20,50,50).Shape()
    aisShape = AIS_ColoredShape(aBox)
    for shll in TopologyExplorer(aBox).shells():
       aisShape.SetCustomColor(shll, color0)
    context.Display(aisShape, True)
    context.SetTransparency(aisShape, 0.8, True)
    owner = aisShape.GetOwner()
    drawer = aisShape.DynamicHilightAttributes()
    context.HilightWithColor(aisShape, drawer, True)
    display.View_Iso()
    display.FitAll()


def onclick_select_surface(shp, *kwargs):
    global context 
    global aisShape
    global whichList, isSelectionEnabled
    global listOne_surfaces,listTwo_surfaces,selected_surfaces
    """ This is the function called every time
    a face is clicked in the 3d view
    """
    one_color = Quantity_Color(1., 0.5, 0.5, 0)
    two_color = Quantity_Color(1., 0.8, 0.2, 0)
    if isSelectionEnabled==False:
        return
    for shape in shp:  # this should be a TopoDS_Face TODO check it is
        print('shape: ',shape,hex(id(shape)))
        for sel_srf in selected_surfaces:
            if shape==sel_srf:
                print('Face already selected')                     
                return
            
        print("Face selected: ", shape)
        if whichList != "hide":
            selected_surfaces.append(shape) # aggiunge la faccia alla lista delle selzionate
        if whichList == "one":
            listOne_surfaces.append(shape) 
            aisShape.SetCustomColor(shape,one_color) #this does not work
            aisShape.SetCustomTransparency(shape, 0.)
            context.HilightWithColor(aisShape, drawer, True)
            #display.DisplayColoredShape(shape,color='RED',update=True) # cambia il colore
        if whichList == "two":
            listTwo_surfaces.append(shape) 
            aisShape.SetCustomColor(shape,two_color) #this does not work
            aisShape.SetCustomTransparency(shape, 0.)
            context.HilightWithColor(aisShape, drawer, True)
        if whichList == "hide":
            hide(shape)
        print('overall number of faces selected:',len(selected_surfaces))
    return 

def one_selection(event=None):
    global listOne_surfaces
    global whichList, isSelectionEnabled
    listOne_surfaces.clear()
    print('Enable face selection one')
    isSelectionEnabled = True
    whichList="one" 
    display.SetSelectionModeFace()  # switch to Face selection mode

def two_selection(event=None):
    global listTwo_surfaces
    global whichList, isSelectionEnabled
    listTwo_surfaces.clear()
    print('Enable face selection two')
    isSelectionEnabled = True
    whichList="two" 
    display.SetSelectionModeFace()  # switch to Face selection mode

def hide_surfaces(event=None):
    global hidden_surfaces
    global whichList, isSelectionEnabled
    print('Enable face hiding')
    isSelectionEnabled = True
    whichList="hide" 
    display.SetSelectionModeFace()  # switch to Face selection mode
    print('isSelectionEnabled',isSelectionEnabled)

def finish_selection():
    global listO_surfaces,listTwo_surfaces,hidden_surfaces
    global whichList, isSelectionEnabled
    print('finish selection')
    isSelectionEnabled = False
    display.SetSelectionMode(None) 
    if whichList == "one":
        print('number of faces selected for ONE:',len(listOne_surfaces))
        for cur_srf in listOne_surfaces:
            hide(cur_srf)
    if whichList == "two":
        print('number of faces selected for HUB:',len(listTwo_surfaces))
        for cur_srf in listTwo_surfaces:
            hide(cur_srf)
    if whichList == "hide":
        print('number of hidden faces:',len(hidden_surfaces))


def hide(aSurface):
    global hidden_surfaces
    global context 
    global aisShape
    ## ISSUE 1 ##
    ## a Surface must be hidden in the display: 
    hidden_surfaces.append(aSurface) 
    print("aSurface was hidden")
    print(type(aSurface))
    aisShape.SetCustomTransparency(aSurface, 1.)
    context.HilightWithColor(aisShape, drawer, True)
    display.FitAll()
    print('')


def show_all(event=None):
    global color0, transparency0
    global aisShape
    global hidden_surfaces
    if len(hidden_surfaces) > 0 :
        for shape   in hidden_surfaces:
           aisShape.SetCustomColor(shape,color0) #this does not work
           aisShape.SetCustomTransparency(shape, transparency0)
           context.HilightWithColor(aisShape, drawer, True)
           ## ISSUE 3 ## qui vorrei mostrare di nuovo le superfici nascoste
        print('')  ##
    else:
         print('No hidden surfaces')


def exit(event=None):
    sys.exit()

if __name__ == '__main__':

    global context 
    global isSelectionEnabled
    isSelectionEnabled=False
    context = display.Context
    context.SetAutoActivateSelection(False)


    display.register_select_callback(onclick_select_surface)
    _qtViewer3d = display._parent 
    print('qtViewer3d',_qtViewer3d)
    _app = _qtViewer3d._qApp
    print('app',_app)
    
    # mapping X to stop selection
    _qtViewer3d._key_map.update({ord('X'):finish_selection}) 
    
    display.View_Iso()
    display.FitAll()


    add_menu('menu')
    add_function_to_menu('menu', box)
    add_function_to_menu('menu', one_selection)
    add_function_to_menu('menu', two_selection)
    add_function_to_menu('menu', exit)
    
    add_menu('Show/Hide')
    add_function_to_menu('Show/Hide',show_all)
    add_function_to_menu('Show/Hide',hide_surfaces)


    start_display()

